@@ -66,85 +66,177 @@ chk(a, b)
 
 
 ###################################################
-### code chunk number 5: ex-subset
+### code chunk number 5: ex-subset (eval = FALSE)
 ###################################################
+## ## subset
+## a <- as.array(ltMatrices(xn, byrow = FALSE, names = nm)[i, j])
+## b <- as.array(ltMatrices(xn, byrow = FALSE, names = nm))[j, j, i]
+## chk(a, b)
+## 
+## a <- as.array(ltMatrices(xn, byrow = TRUE, names = nm)[i, j])
+## b <- as.array(ltMatrices(xn, byrow = TRUE, names = nm))[j, j, i]
+## chk(a, b)
+## 
+## a <- as.array(ltMatrices(xd, byrow = FALSE, 
+##                          diag = TRUE, names = nm)[i, j])
+## b <- as.array(ltMatrices(xd, byrow = FALSE, 
+##                          diag = TRUE, names = nm))[j, j, i]
+## chk(a, b)
+## 
+## a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+##                          names = nm)[i, j])
+## b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+##                          names = nm))[j, j, i]
+## chk(a, b)
+
+
+###################################################
+### code chunk number 6: ex-subset-1
+###################################################
+i <- colnames(xn)[1:2]
+j <- 2:4
 ## subset
-a <- as.array(ltMatrices(xn, byrow = FALSE)[1:2, 2:4])
-b <- as.array(ltMatrices(xn, byrow = FALSE))[2:4, 2:4, 1:2]
+a <- as.array(ltMatrices(xn, byrow = FALSE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = FALSE, names = nm))[j, j, i]
 chk(a, b)
 
-a <- as.array(ltMatrices(xn, byrow = TRUE)[1:2, 2:4])
-b <- as.array(ltMatrices(xn, byrow = TRUE))[2:4, 2:4, 1:2]
+a <- as.array(ltMatrices(xn, byrow = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = TRUE, names = nm))[j, j, i]
 chk(a, b)
 
-a <- as.array(ltMatrices(xd, byrow = FALSE, diag = TRUE)[1:2, 2:4])
-b <- as.array(ltMatrices(xd, byrow = FALSE, diag = TRUE))[2:4, 2:4, 1:2]
+a <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm))[j, j, i]
 chk(a, b)
 
-a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE)[1:2, 2:4])
-b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE))[2:4, 2:4, 1:2]
+a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm))[j, j, i]
 chk(a, b)
 
 
 ###################################################
-### code chunk number 6: ex-subset-2
+### code chunk number 7: ex-subset-2
 ###################################################
+i <- 1:2
+j <- nm[2:4]
 ## subset
+a <- as.array(ltMatrices(xn, byrow = FALSE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = FALSE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xn, byrow = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = TRUE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm))[j, j, i]
+chk(a, b)
+
+
+###################################################
+### code chunk number 8: ex-subset-3
+###################################################
 j <- c(1, 3, 5)
-a <- as.array(ltMatrices(xn, byrow = FALSE)[1:2, j])
-b <- as.array(ltMatrices(xn, byrow = FALSE))[j, j, 1:2]
-chk(a, b)
-
-a <- as.array(ltMatrices(xn, byrow = TRUE)[1:2, j])
-b <- as.array(ltMatrices(xn, byrow = TRUE))[j, j, 1:2]
-chk(a, b)
-
-a <- as.array(ltMatrices(xd, byrow = FALSE, diag = TRUE)[1:2, j])
-b <- as.array(ltMatrices(xd, byrow = FALSE, diag = TRUE))[j, j, 1:2]
-chk(a, b)
-
-a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE)[1:2, j])
-b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE))[j, j, 1:2]
-chk(a, b)
-
-
-###################################################
-### code chunk number 7: ex-subset-3
-###################################################
 ## subset
+a <- as.array(ltMatrices(xn, byrow = FALSE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = FALSE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xn, byrow = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = TRUE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm))[j, j, i]
+chk(a, b)
+
+
+###################################################
+### code chunk number 9: ex-subset-4
+###################################################
+j <- nm[c(1, 3, 5)]
+## subset
+a <- as.array(ltMatrices(xn, byrow = FALSE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = FALSE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xn, byrow = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = TRUE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm))[j, j, i]
+chk(a, b)
+
+a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm))[j, j, i]
+chk(a, b)
+
+
+###################################################
+### code chunk number 10: ex-subset-5
+###################################################
 j <- -c(1, 3, 5)
-a <- as.array(ltMatrices(xn, byrow = FALSE)[1:2, j])
-b <- as.array(ltMatrices(xn, byrow = FALSE))[j, j, 1:2]
+## subset
+a <- as.array(ltMatrices(xn, byrow = FALSE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = FALSE, names = nm))[j, j, i]
 chk(a, b)
 
-a <- as.array(ltMatrices(xn, byrow = TRUE)[1:2, j])
-b <- as.array(ltMatrices(xn, byrow = TRUE))[j, j, 1:2]
+a <- as.array(ltMatrices(xn, byrow = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xn, byrow = TRUE, names = nm))[j, j, i]
 chk(a, b)
 
-a <- as.array(ltMatrices(xd, byrow = FALSE, diag = TRUE)[1:2, j])
-b <- as.array(ltMatrices(xd, byrow = FALSE, diag = TRUE))[j, j, 1:2]
+a <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = FALSE, 
+                         diag = TRUE, names = nm))[j, j, i]
 chk(a, b)
 
-a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE)[1:2, j])
-b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE))[j, j, 1:2]
+a <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm)[i, j])
+b <- as.array(ltMatrices(xd, byrow = TRUE, diag = TRUE, 
+                         names = nm))[j, j, i]
 chk(a, b)
 
 
 ###################################################
-### code chunk number 8: ex-subset-4
+### code chunk number 11: ex-subset-6
 ###################################################
 ## subset
-j <- sample(1:J)
-ltM <- ltMatrices(xn, byrow = FALSE)
-try(ltM[1:2, j])
+j <- nm[sample(1:J)]
+ltM <- ltMatrices(xn, byrow = FALSE, names = nm)
+try(ltM[i, j])
 ltM <- as.syMatrices(ltM)
-a <- as.array(ltM[1:2, j])
-b <- as.array(ltM)[j, j, 1:2]
+a <- as.array(ltM[i, j])
+b <- as.array(ltM)[j, j, i]
 chk(a, b)
 
 
 ###################################################
-### code chunk number 9: ex-Lower_tri
+### code chunk number 12: ex-Lower_tri
 ###################################################
 ## J <- 4
 M <- ltMatrices(matrix(1:10, nrow = 10, ncol = 2), diag = TRUE)
@@ -158,13 +250,13 @@ Lower_tri(invchol2cor(M))
 
 
 ###################################################
-### code chunk number 10: ex-diag
+### code chunk number 13: ex-diag
 ###################################################
 all(diagonals(ltMatrices(xn, byrow = TRUE)) == 1L)
 
 
 ###################################################
-### code chunk number 11: ex-addiag
+### code chunk number 14: ex-addiag
 ###################################################
 lxd2 <- lxn
 diagonals(lxd2) <- 1
@@ -172,7 +264,7 @@ chk(as.array(lxd2), as.array(lxn))
 
 
 ###################################################
-### code chunk number 12: ex-diagJ
+### code chunk number 15: ex-diagJ
 ###################################################
 (I5 <- diagonals(5L))
 diagonals(I5) <- 1:5
@@ -180,7 +272,7 @@ I5
 
 
 ###################################################
-### code chunk number 13: ex-mult
+### code chunk number 16: ex-mult
 ###################################################
 lxn <- ltMatrices(xn, byrow = TRUE)
 lxd <- ltMatrices(xd, byrow = TRUE, diag = TRUE)
@@ -213,7 +305,7 @@ chk(a, b, check.attributes = FALSE)
 
 
 ###################################################
-### code chunk number 14: ex-tmult
+### code chunk number 17: ex-tmult
 ###################################################
 a <- Mult(lxn, y, transpose = TRUE)
 A <- as.array(lxn)
@@ -237,12 +329,13 @@ chk(Mult(lxn, y[,1], transpose = TRUE),
 
 
 ###################################################
-### code chunk number 15: ex-symult
+### code chunk number 18: ex-symult
 ###################################################
 J <- 5
 N1 <- 10
 ex <- expression({
-  C <- syMatrices(matrix(runif(N2 * J * (J + c(-1, 1)[DIAG + 1L] ) / 2), ncol = N2), 
+  C <- syMatrices(matrix(runif(N2 * J * (J + c(-1, 1)[DIAG + 1L] ) / 2),
+                          ncol = N2), 
                   diag = DIAG)
   x <- matrix(runif(N1 * J), nrow = J)
   Ca <- as.array(C)
@@ -267,7 +360,7 @@ eval(ex)
 
 
 ###################################################
-### code chunk number 16: ex-solve
+### code chunk number 19: ex-solve
 ###################################################
 ## solve
 A <- as.array(lxn)
@@ -295,14 +388,14 @@ chk(solve(lxn, y[,1]), solve(lxn, y[,rep(1, N)]))
 
 
 ###################################################
-### code chunk number 17: ex-tsolve
+### code chunk number 20: ex-tsolve
 ###################################################
 chk(solve(lxn[1,], y, transpose = TRUE), 
     t(as.array(solve(lxn[1,]))[,,1]) %*% y)
 
 
 ###################################################
-### code chunk number 18: ex-logdet
+### code chunk number 21: ex-logdet
 ###################################################
 chk(logdet(lxn), colSums(log(diagonals(lxn))))
 chk(logdet(lxd[1,]), colSums(log(diagonals(lxd[1,]))))
@@ -312,7 +405,7 @@ chk(logdet(lxd2), colSums(log(diagonals(lxd2))))
 
 
 ###################################################
-### code chunk number 19: ex-tcrossprod
+### code chunk number 22: ex-tcrossprod
 ###################################################
 ## Tcrossprod
 a <- as.array(Tcrossprod(lxn))
@@ -337,7 +430,7 @@ chk(d, diagonals(Tcrossprod(lxd)))
 
 
 ###################################################
-### code chunk number 20: ex-crossprod
+### code chunk number 23: ex-crossprod
 ###################################################
 ## Crossprod
 a <- as.array(Crossprod(lxn))
@@ -362,7 +455,7 @@ chk(d, diagonals(Crossprod(lxd)))
 
 
 ###################################################
-### code chunk number 21: chol
+### code chunk number 24: chol
 ###################################################
 Sigma <- Tcrossprod(lxd)
 chk(chol(Sigma), lxd)
@@ -372,7 +465,7 @@ chk(as.array(chol(Sigma)), as.array(lxn))
 
 
 ###################################################
-### code chunk number 22: kronecker
+### code chunk number 25: kronecker
 ###################################################
 J <- 10
 
@@ -428,7 +521,7 @@ chk(A, B)
 
 
 ###################################################
-### code chunk number 23: conv-ex-1
+### code chunk number 26: conv-ex-1
 ###################################################
 prec2pc <- function(x) {
     ret <- -cov2cor(x)
@@ -455,7 +548,7 @@ chk(unlist(PC), c(as.array(invchol2pc(L))),
 
 
 ###################################################
-### code chunk number 24: conv-ex-2
+### code chunk number 27: conv-ex-2
 ###################################################
 C <- lxn
 Sigma <- apply(as.array(C), 3, 
@@ -477,7 +570,7 @@ chk(unlist(PC), c(as.array(chol2pc(C))),
 
 
 ###################################################
-### code chunk number 25: conv-ex-3
+### code chunk number 28: conv-ex-3
 ###################################################
 L <- lxd
 Sigma <- apply(as.array(L), 3, 
@@ -499,7 +592,7 @@ chk(unlist(PC), c(as.array(invchol2pc(L))),
 
 
 ###################################################
-### code chunk number 26: conv-ex-4
+### code chunk number 29: conv-ex-4
 ###################################################
 C <- lxd
 Sigma <- apply(as.array(C), 3, 
@@ -521,21 +614,21 @@ chk(unlist(PC), c(as.array(chol2pc(C))),
 
 
 ###################################################
-### code chunk number 27: aperm-tests
+### code chunk number 30: aperm-tests
 ###################################################
-L <- lxn
+L <- as.invchol(lxn)
 J <- dim(L)[2L]
-Lp <- aperm(a = L, perm = p <- sample(1:J), is_chol = FALSE)
+Lp <- aperm(a = L, perm = p <- sample(1:J))
 chk(invchol2cov(L)[,p], invchol2cov(Lp))
 
-C <- lxn
+C <- as.chol(lxn)
 J <- dim(C)[2L]
-Cp <- aperm(a = C, perm = p <- sample(1:J), is_chol = TRUE)
+Cp <- aperm(a = C, perm = p <- sample(1:J))
 chk(chol2cov(C)[,p], chol2cov(Cp))
 
 
 ###################################################
-### code chunk number 28: marg
+### code chunk number 31: marg
 ###################################################
 Sigma <- Tcrossprod(lxd)
 j <- 1:3
@@ -551,7 +644,7 @@ chk(Sigma[,j], Tcrossprod(solve(marg_mvnorm(invchol = lxd, which = j)$invchol)))
 
 
 ###################################################
-### code chunk number 29: cond-general
+### code chunk number 32: cond-general
 ###################################################
 Sigma <- as.array(Tcrossprod(lxd))[,,1]
 j <- 2:4
@@ -581,7 +674,7 @@ chk(cS, as.array(Tcrossprod(solve(cmv$invchol)))[,,1])
 
 
 ###################################################
-### code chunk number 30: cond-simple
+### code chunk number 33: cond-simple
 ###################################################
 Sigma <- as.array(Tcrossprod(lxd))[,,1]
 j <- 1:3
@@ -611,7 +704,7 @@ chk(cS, as.array(Tcrossprod(solve(cmv$invchol)))[,,1])
 
 
 ###################################################
-### code chunk number 31: ex-MV
+### code chunk number 34: ex-MV
 ###################################################
 N <- 1000L
 J <- 50L
@@ -622,19 +715,20 @@ Y <- solve(lt, Z)
 ll1 <- sum(dnorm(Mult(lt, Y), log = TRUE)) + sum(log(diagonals(lt)))
 
 S <- as.array(Tcrossprod(solve(lt)))
-ll2 <- sum(sapply(1:N, function(i) dmvnorm(x = Y[,i], sigma = S[,,i], log = TRUE)))
+ll2 <- sum(sapply(1:N, function(i) 
+                           dmvnorm(x = Y[,i], sigma = S[,,i], log = TRUE)))
 chk(ll1, ll2)
 
 
 ###################################################
-### code chunk number 32: ex-MV-d
+### code chunk number 35: ex-MV-d
 ###################################################
 ll3 <- ldmvnorm(obs = Y, invchol = lt)
 chk(ll1, ll3)
 
 
 ###################################################
-### code chunk number 33: ex-MV-mc
+### code chunk number 36: ex-MV-mc
 ###################################################
 ## marginal of and conditional on these
 (j <- 1:5 * 10)
@@ -649,13 +743,13 @@ chk(ll1, ll3)
 
 
 ###################################################
-### code chunk number 34: chapterseed
+### code chunk number 37: chapterseed
 ###################################################
 set.seed(270312)
 
 
 ###################################################
-### code chunk number 35: fct-lpmvnormR
+### code chunk number 38: fct-lpmvnormR
 ###################################################
 
 lpmvnormR <- function(lower, upper, mean = 0, center = NULL, chol, logLik = TRUE, ...) {
@@ -665,7 +759,7 @@ lpmvnormR <- function(lower, upper, mean = 0, center = NULL, chol, logLik = TRUE
     if (!is.matrix(upper)) upper <- matrix(upper, ncol = 1)
     stopifnot(isTRUE(all.equal(dim(lower), dim(upper))))
 
-    stopifnot(inherits(chol, "ltMatrices"))
+    stopifnot(is.ltMatrices(chol))		### NOTE: replace with is.chol
     byrow_orig <- attr(chol, "byrow")
     chol <- ltMatrices(chol, byrow = TRUE)
     d <- dim(chol)
@@ -675,8 +769,11 @@ lpmvnormR <- function(lower, upper, mean = 0, center = NULL, chol, logLik = TRUE
 
     stopifnot(nrow(lower) == J && ncol(lower) == N)
     stopifnot(nrow(upper) == J && ncol(upper) == N)
-    if (is.matrix(mean))
+    if (is.matrix(mean)) {
+        if (ncol(mean) == 1L) 
+            mean <- mean[,rep(1, N),drop = FALSE]
         stopifnot(nrow(mean) == J && ncol(mean) == N)
+    }
 
     lower <- lower - mean
     upper <- upper - mean
@@ -709,7 +806,7 @@ lpmvnormR <- function(lower, upper, mean = 0, center = NULL, chol, logLik = TRUE
 
 
 ###################################################
-### code chunk number 36: ex-lpmvnorm_R
+### code chunk number 39: ex-lpmvnorm_R
 ###################################################
 J <- 5L
 N <- 10L
@@ -726,7 +823,7 @@ b[sample(J * N)[1:2]] <- Inf
 
 
 ###################################################
-### code chunk number 37: ex-again
+### code chunk number 40: ex-again
 ###################################################
 phat
 exp(lpmvnorm(a, b, chol = lx, M = 25000, logLik = FALSE, fast = TRUE))
@@ -734,7 +831,7 @@ exp(lpmvnorm(a, b, chol = lx, M = 25000, logLik = FALSE, fast = FALSE))
 
 
 ###################################################
-### code chunk number 38: ex-lpmvnorm
+### code chunk number 41: ex-lpmvnorm
 ###################################################
 M <- 10000L
 if (require("qrng", quietly = TRUE)) {
@@ -745,7 +842,7 @@ if (require("qrng", quietly = TRUE)) {
     W <- matrix(runif(M * (J - 1)), nrow = J - 1)
 }
 
-### Genz & Bretz, 2001, without early stopping (really?)
+### Genz & Bretz, 2002, without early stopping (really?)
 pGB <- lpmvnormR(a, b, chol = lx, logLik = FALSE, 
                 algorithm = GenzBretz(maxpts = M, abseps = 0, releps = 0))
 ### Genz 1992 with quasi-Monte-Carlo, fast pnorm
@@ -765,7 +862,7 @@ cbind(pGB, pGqf, pGf, pGqs, pGs)
 
 
 ###################################################
-### code chunk number 39: ex-uni
+### code chunk number 42: ex-uni
 ###################################################
 ### test univariate problem
 ### call pmvnorm
@@ -782,7 +879,7 @@ cbind(c(ptr), pGB, pGq)
 
 
 ###################################################
-### code chunk number 40: ex-score
+### code chunk number 43: ex-score
 ###################################################
 J <- 5L
 N <- 4L
@@ -813,11 +910,12 @@ sC <- slpmvnorm(a, b, chol = mC, w = W, M = M)
 chk(lli, sC$logLik)
 
 if (require("numDeriv", quietly = TRUE))
-    chk(grad(fC, unclass(mC)), rowSums(unclass(sC$chol)), check.attributes = FALSE)
+    chk(grad(fC, unclass(mC)), rowSums(unclass(sC$chol)), 
+        check.attributes = FALSE)
 
 
 ###################################################
-### code chunk number 41: ex-Lscore
+### code chunk number 44: ex-Lscore
 ###################################################
 mL <- solve(mC)
 
@@ -840,25 +938,25 @@ if (require("numDeriv", quietly = TRUE))
 
 
 ###################################################
-### code chunk number 42: ex-uni-score
+### code chunk number 45: ex-uni-score
 ###################################################
 ptr <- pnorm(b[1,] / c(unclass(mC[,1]))) - pnorm(a[1,] / c(unclass(mC[,1])))
 log(ptr)
 lpmvnorm(a[1,,drop = FALSE], b[1,,drop = FALSE], chol = mC[,1], logLik = FALSE)
-lapply(slpmvnorm(a[1,,drop = FALSE], b[1,,drop = FALSE], chol = mC[,1], logLik =
-TRUE), unclass)
+lapply(slpmvnorm(a[1,,drop = FALSE], b[1,,drop = FALSE], chol = mC[,1], 
+                 logLik = TRUE), unclass)
 sd1 <- c(unclass(mC[,1]))
 (dnorm(b[1,] / sd1) * b[1,] - dnorm(a[1,] / sd1) * a[1,]) * (-1) / sd1^2 / ptr
 
 
 ###################################################
-### code chunk number 43: chapterseed
+### code chunk number 46: chapterseed
 ###################################################
 set.seed(110515)
 
 
 ###################################################
-### code chunk number 44: ex-ML-dgp
+### code chunk number 47: ex-ML-dgp
 ###################################################
 J <- 4
 R <- diag(J)
@@ -871,7 +969,7 @@ round(Sigma <- diag(sqrt(1:J / 2)) %*% R %*% diag(sqrt(1:J / 2)), 7)
 
 
 ###################################################
-### code chunk number 45: ex-ML-C
+### code chunk number 48: ex-ML-C
 ###################################################
 prm <- C[lower.tri(C, diag = TRUE)]
 lt <- ltMatrices(matrix(prm, ncol = 1L), 
@@ -885,7 +983,7 @@ chk(Sigma, as.array(Tcrossprod(lt))[,,1], check.attributes = FALSE)
 
 
 ###################################################
-### code chunk number 46: ex-ML-data
+### code chunk number 49: ex-ML-data
 ###################################################
 N <- 100L
 Z <- matrix(rnorm(N * J), nrow = J)
@@ -893,14 +991,14 @@ Y <- Mult(lt, Z) + (mn <- 1:J)
 
 
 ###################################################
-### code chunk number 47: ex-ML-mu-vcov
+### code chunk number 50: ex-ML-mu-vcov
 ###################################################
 rowMeans(Y)
 (Shat <- var(t(Y)) * (N - 1) / N)
 
 
 ###################################################
-### code chunk number 48: ex-ML-clogLik
+### code chunk number 51: ex-ML-clogLik
 ###################################################
 Yc <- Y - rowMeans(Y)
 
@@ -916,14 +1014,14 @@ sc <- function(parm) {
 
 
 ###################################################
-### code chunk number 49: ex-ML-const
+### code chunk number 52: ex-ML-const
 ###################################################
 llim <- rep(-Inf, J * (J + 1) / 2)
 llim[which(rownames(unclass(lt)) %in% paste(1:J, 1:J, sep = "."))] <- 1e-4
 
 
 ###################################################
-### code chunk number 50: ex-ML-c
+### code chunk number 53: ex-ML-c
 ###################################################
 if (BYROW) {
   cML <- chol(Shat)[upper.tri(Shat, diag = TRUE)]
@@ -937,7 +1035,7 @@ if (require("numDeriv", quietly = TRUE))
 
 
 ###################################################
-### code chunk number 51: ex-ML-coptim
+### code chunk number 54: ex-ML-coptim
 ###################################################
 op <- optim(start, fn = ll, gr = sc, method = "L-BFGS-B", 
             lower = llim, control = list(trace = TRUE))
@@ -952,7 +1050,7 @@ lt
 
 
 ###################################################
-### code chunk number 52: ex-ML-cens
+### code chunk number 55: ex-ML-cens
 ###################################################
 prb <- 1:9 / 10
 sds <- sqrt(diag(Sigma))
@@ -966,7 +1064,7 @@ for (j in 1:J) {
 
 
 ###################################################
-### code chunk number 53: ex-ML-chk (eval = FALSE)
+### code chunk number 56: ex-ML-chk (eval = FALSE)
 ###################################################
 ## M <- floor(exp(0:25/10) * 1000)
 ## lGB <- sapply(M, function(m) {
@@ -994,7 +1092,7 @@ for (j in 1:J) {
 
 
 ###################################################
-### code chunk number 54: ex-ML-fig-data
+### code chunk number 57: ex-ML-fig-data
 ###################################################
 ### use pre-computed data, otherwise CRAN complains.
 M <-
@@ -1031,7 +1129,7 @@ rownames(lHf) <- c("user.self", "ll")
 
 
 ###################################################
-### code chunk number 55: ex-ML-fig
+### code chunk number 58: ex-ML-fig
 ###################################################
 layout(matrix(1:2, nrow = 1))
 plot(M, lGB["ll",], ylim = range(c(lGB["ll",], lH["ll",], lHf["ll",])), ylab = "Log-likelihood")
@@ -1044,7 +1142,7 @@ legend("bottomright", legend = c("pmvnorm", "lpmvnorm", "lpmvnorm(fast)"), pch =
 
 
 ###################################################
-### code chunk number 56: ex-ML-ll
+### code chunk number 59: ex-ML-ll
 ###################################################
 M <- 500 
 if (require("qrng", quietly = TRUE)) {
@@ -1055,8 +1153,8 @@ if (require("qrng", quietly = TRUE)) {
     W <- matrix(runif(M * (J - 1)), nrow = J - 1)
 }
 ll <- function(parm, J) {
-     m <- parm[1:J]		### mean parameters
-     parm <- parm[-(1:J)]	### chol parameters
+     m <- parm[1:J]             ### mean parameters
+     parm <- parm[-(1:J)]       ### chol parameters
      C <- matrix(c(parm), ncol = 1L)
      C <- ltMatrices(C, diag = TRUE, byrow = BYROW)
      -lpmvnorm(lower = lwr, upper = upr, mean = m, chol = C, 
@@ -1065,7 +1163,7 @@ ll <- function(parm, J) {
 
 
 ###################################################
-### code chunk number 57: ex-ML-check
+### code chunk number 60: ex-ML-check
 ###################################################
 prm <- c(mn, unclass(lt))
 ll(prm, J = J)
@@ -1078,7 +1176,7 @@ chk(llprm, sum(lpmvnorm(lwr, upr, mean = mn, chol = lt, w = W,
 
 
 ###################################################
-### code chunk number 58: ex-ML-sc
+### code chunk number 61: ex-ML-sc
 ###################################################
 sc <- function(parm, J) {
     m <- parm[1:J]             ### mean parameters
@@ -1092,14 +1190,14 @@ sc <- function(parm, J) {
 
 
 ###################################################
-### code chunk number 59: ex-ML-sc-chk
+### code chunk number 62: ex-ML-sc-chk
 ###################################################
 if (require("numDeriv", quietly = TRUE))
     chk(grad(ll, prm, J = J), sc(prm, J = J), check.attributes = FALSE)
 
 
 ###################################################
-### code chunk number 60: ex-ML
+### code chunk number 63: ex-ML
 ###################################################
 llim <- rep(-Inf, J + J * (J + 1) / 2)
 llim[J + which(rownames(unclass(lt)) %in% paste(1:J, 1:J, sep = "."))] <- 1e-4
@@ -1120,7 +1218,7 @@ ll(prm, J = J)
 
 
 ###################################################
-### code chunk number 61: ex-ML-C
+### code chunk number 64: ex-ML-C
 ###################################################
 (C <- ltMatrices(matrix(op$par[-(1:J)], ncol = 1), 
                  diag = TRUE, byrow = BYROW))
@@ -1128,14 +1226,14 @@ lt
 
 
 ###################################################
-### code chunk number 62: ex-ML-mu
+### code chunk number 65: ex-ML-mu
 ###################################################
 op$par[1:J]
 mn
 
 
 ###################################################
-### code chunk number 63: ex-ML-Shat
+### code chunk number 66: ex-ML-Shat
 ###################################################
 ### ATLAS print issues
 round(Tcrossprod(lt), 7)  ### true Sigma
@@ -1144,13 +1242,27 @@ round(Shat, 7)            ### "exact" obs
 
 
 ###################################################
-### code chunk number 64: regressions
+### code chunk number 67: regressions
 ###################################################
 c(cond_mvnorm(chol = C, which = 2:J, given = diag(J - 1))$mean)
 
 
 ###################################################
-### code chunk number 65: lm-ex
+### code chunk number 68: regressionsC
+###################################################
+c(cond_mvnorm(chol = aperm(as.chol(C), perm = c(2:J, 1)),
+              which = 1:(J - 1), given = diag(J - 1))$mean)
+
+
+###################################################
+### code chunk number 69: regressionsP
+###################################################
+x <- as.array(chol2pre(aperm(as.chol(C), perm = c(2:J, 1))))[J,,1]
+c(-x[-J] / x[J])
+
+
+###################################################
+### code chunk number 70: lm-ex
 ###################################################
 dY <- as.data.frame(t(Y))
 colnames(dY) <- paste0("Y", 1:J)
@@ -1158,14 +1270,14 @@ coef(m1 <- lm(Y1 ~ ., data = dY))[-1L]
 
 
 ###################################################
-### code chunk number 66: hessian
+### code chunk number 71: hessian
 ###################################################
 H <- optim(op$par, fn = ll, gr = sc, J = J, method = "L-BFGS-B", 
            lower = llim, hessian = TRUE)$hessian
 
 
 ###################################################
-### code chunk number 67: ML-sample
+### code chunk number 72: ML-sample
 ###################################################
 L <- try(t(chol(H)))
 ### some check on r-oldrel-macos-arm64
@@ -1178,57 +1290,59 @@ rC <- solve(L, Z)[-(1:J),] + op$par[-(1:J)] ### remove mean parameters
 
 
 ###################################################
-### code chunk number 68: ML-check
+### code chunk number 73: ML-check
 ###################################################
 c(sqrt(rowMeans((rC - rowMeans(rC))^2)))
 c(sqrt(diagonals(Crossprod(solve(L)))))
 
 
 ###################################################
-### code chunk number 69: rC
+### code chunk number 74: rC
 ###################################################
 rC <- ltMatrices(rC, diag = TRUE)
 
 
 ###################################################
-### code chunk number 70: ML-beta
+### code chunk number 75: ML-beta
 ###################################################
 rbeta <- cond_mvnorm(chol = rC, which = 2:J, given = diag(J - 1))$mean
 sqrt(rowMeans((rbeta - rowMeans(rbeta))^2))
 
 
 ###################################################
-### code chunk number 71: se-ex
+### code chunk number 76: se-ex
 ###################################################
 sqrt(diag(vcov(m1)))[-1L]
 
 
 ###################################################
-### code chunk number 72: ex-ML-cd
+### code chunk number 77: ex-ML-cd
 ###################################################
+ic <- 1:2 	### position of continuous variables
 ll_cd <- function(parm, J) {
      m <- parm[1:J]             ### mean parameters
      parm <- parm[-(1:J)]       ### chol parameters
      C <- matrix(c(parm), ncol = 1L)
      C <- ltMatrices(C, diag = TRUE, byrow = BYROW)
-     -ldpmvnorm(obs = Y[1:2,], lower = lwr[-(1:2),], 
-                upper = upr[-(1:2),], mean = m, chol = C, 
-                w = W[-(1:2),,drop = FALSE], M = M)
+     -ldpmvnorm(obs = Y[ic,], lower = lwr[-ic,], 
+                upper = upr[-ic,], mean = m, chol = C, 
+                w = W[-ic,,drop = FALSE], M = M)
 }
 sc_cd <- function(parm, J) {
     m <- parm[1:J]             ### mean parameters
     parm <- parm[-(1:J)]       ### chol parameters
     C <- matrix(c(parm), ncol = 1L)
     C <- ltMatrices(C, diag = TRUE, byrow = BYROW)
-    ret <- sldpmvnorm(obs = Y[1:2,], lower = lwr[-(1:2),],
-                      upper = upr[-(1:2),], mean = m, chol = C, 
-                      w = W[-(1:2),,drop = FALSE], M = M)
-    return(-c(rowSums(ret$mean), rowSums(unclass(ret$chol))))
+    ret <- sldpmvnorm(obs = Y[ic,], lower = lwr[-ic,],
+                      upper = upr[-ic,], mean = m, chol = C, 
+                      w = W[-ic,,drop = FALSE], M = M)
+    return(-c(rowSums(ret$mean),
+              rowSums(Lower_tri(ret$chol, diag = TRUE))))
 }
 
 
 ###################################################
-### code chunk number 73: ex-ML-cd-score
+### code chunk number 78: ex-ML-cd-score
 ###################################################
 if (require("numDeriv", quietly = TRUE))
     chk(grad(ll_cd, start, J = J), sc_cd(start, J = J), 
@@ -1236,7 +1350,7 @@ if (require("numDeriv", quietly = TRUE))
 
 
 ###################################################
-### code chunk number 74: ex-ML-cd-optim
+### code chunk number 79: ex-ML-cd-optim
 ###################################################
 op <- optim(start, fn = ll_cd, gr = sc_cd, J = J, 
             method = "L-BFGS-B", lower = llim, 
@@ -1253,7 +1367,66 @@ mn
 
 
 ###################################################
-### code chunk number 75: ex-stand
+### code chunk number 80: ex-ML-ap
+###################################################
+### discrete variables first
+perm <- c((1:J)[-ic], ic)
+ll_ap <- function(parm, J) {
+     m <- parm[1:J]             ### mean parameters; NOT permuted
+     parm <- parm[-(1:J)]       ### chol parameters
+     C <- matrix(c(parm), ncol = 1L)
+     C <- ltMatrices(C, diag = TRUE, byrow = BYROW)
+     Ct <- aperm(as.chol(C), perm = perm)
+     -ldpmvnorm(obs = Y[ic,], lower = lwr[-ic,], 
+                upper = upr[-ic,], mean = m, chol = Ct, 
+                w = W[-ic,,drop = FALSE], M = M)
+}
+
+
+###################################################
+### code chunk number 81: ex-ML-ap-score
+###################################################
+sc_ap <- function(parm, J) {
+    m <- parm[1:J]               ### mean parameters; NOT permuted
+    parm <- parm[-(1:J)]         ### chol parameters
+    C <- matrix(c(parm), ncol = 1L)
+    C <- ltMatrices(C, diag = TRUE, byrow = BYROW)
+    ### permutation
+    Ct <- aperm(as.chol(C), perm = perm)
+    ret <- sldpmvnorm(obs = Y[ic,], lower = lwr[-ic,],
+                      upper = upr[-ic,], mean = m, chol = Ct, 
+                      w = W[-ic,,drop = FALSE], M = M)
+    ### undo permutation for chol
+    retC <- deperma(chol = C, permuted_chol = Ct, 
+                   perm = perm, score_schol = ret$chol)
+    return(-c(rowSums(ret$mean),
+              rowSums(Lower_tri(retC, diag = TRUE))))
+}
+
+
+###################################################
+### code chunk number 82: ex-ML-ap-grad
+###################################################
+if (require("numDeriv", quietly = TRUE))
+    chk(grad(ll_ap, start, J = J), sc_ap(start, J = J), 
+        check.attributes = FALSE, tol = 1e-6)
+
+
+###################################################
+### code chunk number 83: ex-ML-ap-optim-
+###################################################
+op <- optim(start, fn = ll_ap, gr = sc_ap, J = J, 
+            method = "L-BFGS-B", lower = llim, 
+            control = list(trace = TRUE))
+## estimated C for (X, Y)
+ltMatrices(matrix(op$par[-(1:J)], ncol = 1), 
+           diag = TRUE, byrow = BYROW)
+## compare with true _permuted_ C for (X, Y)
+aperm(as.chol(lt), perm = perm)
+
+
+###################################################
+### code chunk number 84: ex-stand
 ###################################################
 C <- ltMatrices(runif(10))
 all.equal(as.array(chol2cov(standardize(chol = C))),
@@ -1264,9 +1437,9 @@ all.equal(as.array(invchol2cov(standardize(invchol = L))),
 
 
 ###################################################
-### code chunk number 76: gc-classical
+### code chunk number 85: gc-classical
 ###################################################
-data("iris")
+data("iris", package = "datasets")
 J <- 4
 Z <- t(qnorm(do.call("cbind", lapply(iris[1:J], rank)) / (nrow(iris) + 1)))
 (CR <- cor(t(Z)))
@@ -1291,7 +1464,7 @@ S_ML <- chol2cov(standardize(ltMatrices(op$par)))
 
 
 ###################################################
-### code chunk number 77: gc-NPML
+### code chunk number 86: gc-NPML
 ###################################################
 lwr <- do.call("cbind", lapply(iris[1:J], rank, ties.method = "min")) - 1L
 upr <- do.call("cbind", lapply(iris[1:J], rank, ties.method = "max"))
@@ -1326,14 +1499,14 @@ S_NPML <- chol2cov(standardize(ltMatrices(op2$par)))
 
 
 ###################################################
-### code chunk number 78: gc
+### code chunk number 87: gc
 ###################################################
 S_ML
 S_NPML
 
 
 ###################################################
-### code chunk number 79: gc-se
+### code chunk number 88: gc-se
 ###################################################
 sd_ML <- ltMatrices(sqrt(diag(solve(op$hessian))))
 diagonals(sd_ML) <- 0
@@ -1345,3 +1518,114 @@ if (!inherits(sd_NPML, "try-error")) {
 }
 
 
+###################################################
+### code chunk number 89: iris-model
+###################################################
+data("iris", package = "datasets")
+vars <- names(iris)[-5L]
+m <- colMeans(iris[,vars])
+V <- var(iris[,vars])
+iris_mvn <- mvnorm(mean = m, chol = t(chol(V)))
+iris_var <- simulate(iris_mvn, nsim = nrow(iris))
+
+
+###################################################
+### code chunk number 90: iris-mc
+###################################################
+j <- 3:4
+margDist(iris_mvn, which = vars[j])
+gm <- t(iris[,vars[-(j)]])
+iris_cmvn <- condDist(iris_mvn, which = vars[j], given = gm)
+
+
+###################################################
+### code chunk number 91: iris-ll
+###################################################
+logLik(object = iris_cmvn, obs = t(iris[,vars[-j]]))
+
+
+###################################################
+### code chunk number 92: iris-ll-perm
+###################################################
+logLik(object = iris_cmvn, obs = t(iris[,rev(vars[-j])]))
+
+
+###################################################
+### code chunk number 93: iris-lLgrad
+###################################################
+J <- length(vars)
+obs <- t(iris[, vars])
+lower <- upper <- NULL
+ll <- function(parm) {
+    C <- ltMatrices(parm[-(1:J)], diag = TRUE, names = vars)
+    x <- mvnorm(mean = parm[1:J], chol = C)
+    -logLik(object = x, obs = obs, lower = lower, upper = upper)
+}
+sc <- function(parm) {
+    C <- ltMatrices(parm[-(1:J)], diag = TRUE, names = vars)
+    x <- mvnorm(mean = parm[1:J], chol = C)
+    ret <- lLgrad(object = x, obs = obs, lower = lower, upper = upper)
+    -c(rowSums(ret$mean), rowSums(Lower_tri(ret$scale, diag = TRUE)))
+}
+
+
+###################################################
+### code chunk number 94: iris-ML
+###################################################
+start <- c(c(iris_mvn$mean), Lower_tri(iris_mvn$scale, diag = TRUE))
+if (require("numDeriv", quietly = TRUE))
+    chk(grad(ll, start), sc(start), check.attributes = FALSE)
+op <- optim(start, fn = ll, gr = sc, method = "L-BFGS-B", 
+            lower = llim, control = list(trace = TRUE))
+Chat <- ltMatrices(op$par[-(1:J)], diag = TRUE, names = vars)
+ML <- mvnorm(mean = op$par[1:J], chol = Chat)
+
+
+###################################################
+### code chunk number 95: iris-ML-hat
+###################################################
+### covariance
+round(chol2cov(ML$scale), 2)
+N <- nrow(iris)
+round(V * (N - 1) / N, 2)
+### mean
+ML$mean[,,drop = TRUE]
+m
+
+
+###################################################
+### code chunk number 96: iris-interval
+###################################################
+v1 <- vars[1]
+q1 <- quantile(iris[[v1]], prob = 1:4 / 5)
+head(f1 <- cut(iris[[v1]], breaks = c(-Inf, q1, Inf)))
+
+
+###################################################
+### code chunk number 97: iris-MLi
+###################################################
+lower <- matrix(c(-Inf, q1)[f1], nrow = 1)
+upper <- matrix(c(q1, Inf)[f1], nrow = 1)
+rownames(lower) <- rownames(upper) <- v1
+obs <- obs[!rownames(obs) %in% v1,,drop = FALSE]
+if (require("numDeriv", quietly = TRUE))
+    chk(grad(ll, start), sc(start), check.attributes = FALSE)
+opi <- optim(start, fn = ll, gr = sc, method = "L-BFGS-B", 
+             lower = llim, control = list(trace = TRUE))
+Chati <- ltMatrices(opi$par[-(1:J)], diag = TRUE, names = vars)
+MLi <- mvnorm(mean = opi$par[1:J], chol = Chati)
+
+
+###################################################
+### code chunk number 98: iris-MLi-hat
+###################################################
+op$value
+opi$value
+### covariance
+round(chol2cov(MLi$scale), 2)
+round(chol2cov(ML$scale), 2)
+### mean
+MLi$mean[,,drop = TRUE]
+ML$mean[,,drop = TRUE]
+
+
